const gql = require('graphql-tag')

const schema = gql`
  extend type Query {
    "Introspect GraphQL server itself"
    introspect: Introspect
  }

  "Introspect contains basic information about the GraphQL server itself"
  type Introspect {
    "Self name"
    name: String!

    "Self version"
    version: String!

    "Self setup"
    setup: Object!

    "Self report"
    report: Object
  }

`


const resolvers = {
  Query: {
    introspect: (_source, _args, {dataSources} ) => {
      if (dataSources.introspect) {
        return {}
      }
    },
  },
  Introspect: {
    name(_source, _args, context) {
      if (context.dataSources.introspect) return context.dataSources.introspect.getName()
    },
    version(_source, _args, context) {
      if (context.dataSources.introspect) return context.dataSources.introspect.getVersion()
    },
    setup(_source, _args, context) {
      if (context.dataSources.introspect) return context.dataSources.introspect.getSetup()
    },
    report(_source, _args, context) {
      if (context.dataSources.introspect) return context.dataSources.introspect.getReport(context)
    },
  }
}


const _ = require('lodash')

const sortobject = require('deep-sort-object')
// const { debugger } = require('yves')
const reportTimeout = 1000

class Introspect {
  constructor(obj) {
    this.set(obj)
  }

  set({name,version,setup}) {
    if (name) this.name = name
    if (version) this.version = version
    if (setup) this.setup = setup
  }

  getName() {
    return this.name
  }

  getVersion() {
    return this.version
  }

  getSetup() {
    return this.setup || {}
  }

  async getReport(context) {
    return new Promise( (resolve) => {
      setTimeout( () => {
        // debug('getReport timeout')
        resolve(sortobject(context.res.report)) 
      }, reportTimeout) /* ensures resolving after no reporting within time limit */
    })
  }
}

module.exports = {
  schema,
  resolvers,
  datasource: Introspect
}
